@@ -1,6 +1,8 @@
 from typing import Any, Callable, List, Optional, Dict, Union
 
-import numpy as np  # type: ignore
+import tensorflow as tf  # type: ignore
+import numpy as np
+import numpy.typing as npt
 from dataclasses import dataclass, field
 
 from code_loader.contract.decoder_classes import LeapImage, LeapText, LeapNumeric, LeapGraph, LeapHorizontalBar, \
@@ -16,7 +18,7 @@ class PreprocessResponse:
     data: Any
 
 
-SectionCallableInterface = Callable[[int, PreprocessResponse], np.ndarray]
+SectionCallableInterface = Callable[[int, PreprocessResponse], npt.NDArray[np.float32]]
 
 
 @dataclass
@@ -39,12 +41,21 @@ DecoderCallableReturnType = Union[LeapImage, LeapNumeric, LeapText,
                                   LeapGraph, LeapHorizontalBar, LeapImageMask, LeapTextMask]
 
 
+CustomLossCallableInterface = Callable[[tf.Tensor, tf.Tensor], tf.Tensor]
+
+
+@dataclass
+class CustomLossHandler:
+    name: str
+    function: CustomLossCallableInterface
+
+
 @dataclass
 class DecoderHandler:
     name: str
     function: DecoderCallableInterface
     type: LeapDataType
-    heatmap_function: Optional[Callable[..., np.array]] = None
+    heatmap_function: Optional[Callable[..., npt.NDArray[np.float32]]] = None
 
 
 @dataclass
@@ -76,12 +87,13 @@ class DatasetIntegrationSetup:
     ground_truths: List[GroundTruthHandler] = field(default_factory=list)
     metadata: List[MetadataHandler] = field(default_factory=list)
     prediction_types: List[PredictionTypeInstance] = field(default_factory=list)
+    custom_loss_handlers: List[CustomLossHandler] = field(default_factory=list)
 
 
 @dataclass
 class DatasetSample:
-    inputs: Dict[str, np.ndarray]
-    gt: Dict[str, np.ndarray]
-    metadata: Dict[str, np.ndarray]
+    inputs: Dict[str, npt.NDArray[np.float32]]
+    gt: Dict[str, npt.NDArray[np.float32]]
+    metadata: Dict[str, npt.NDArray[np.float32]]
     index: int
     state: DataStateEnum
