@@ -1,10 +1,11 @@
 from functools import lru_cache
 from typing import Dict, List, Iterable, Any
 
-import numpy as np  # type: ignore
+import numpy as np
+import numpy.typing as npt
 
 from code_loader.contract.datasetclasses import DatasetSample, DatasetBaseHandler, InputHandler, \
-    GroundTruthHandler, PreprocessResponse, DecoderHandler, DecoderCallableReturnType
+    GroundTruthHandler, PreprocessResponse, DecoderHandler, DecoderCallableReturnType, CustomLossHandler
 from code_loader.contract.enums import DataStateEnum, TestingSectionEnum, DataStateType
 from code_loader.contract.responsedataclasses import DatasetIntegParseResult, DatasetTestResultPayload, \
     DatasetPreprocess, DatasetSetup, DatasetInputInstance, DatasetOutputInstance, DatasetMetadataInstance, \
@@ -35,6 +36,15 @@ class DatasetLoader:
             for decoder_handler in setup.decoders
         }
 
+    @lru_cache()
+    def custom_loss_by_name(self) -> Dict[str, CustomLossHandler]:
+        self.exec_script()
+        setup = global_dataset_binder.setup_container
+        return {
+            custom_loss_handler.name: custom_loss_handler
+            for custom_loss_handler in setup.custom_loss_handlers
+        }
+
     def get_sample(self, state: DataStateEnum, idx: int) -> DatasetSample:
         self.exec_script()
         sample = DatasetSample(inputs=self._get_inputs(state, idx),
@@ -121,11 +131,11 @@ class DatasetLoader:
         all_dataset_base_handlers.extend(global_dataset_binder.setup_container.metadata)
         return all_dataset_base_handlers
 
-    def run_decoder(self, decoder_name: str, input_tensors: List[np.array],
+    def run_decoder(self, decoder_name: str, input_tensors: List[npt.NDArray[np.float32]],
                     ) -> DecoderCallableReturnType:
         return self.decoder_by_name()[decoder_name].function(*input_tensors)
 
-    def run_heatmap_decoder(self, decoder_name: str, input_heatmaps: List[np.array]) -> np.array:
+    def run_heatmap_decoder(self, decoder_name: str, input_heatmaps: List[npt.NDArray[np.float32]]) -> npt.NDArray[np.float32]:
         heatmap_function = self.decoder_by_name()[decoder_name].heatmap_function
         if heatmap_function is None:
             assert len(input_heatmaps) == 1
@@ -160,8 +170,11 @@ class DatasetLoader:
         decoders = [DecoderInstance(name=decoder_handler.name, type=decoder_handler.type)
                     for decoder_handler in setup.decoders]
 
+        custom_loss_names = [custom_loss.name for custom_loss in setup.custom_loss_handlers]
+
         return DatasetSetup(preprocess=dataset_preprocess, inputs=inputs, outputs=ground_truths, metadata=metadata,
-                            decoders=decoders, prediction_types=setup.prediction_types)
+                            decoders=decoders, prediction_types=setup.prediction_types,
+                            custom_loss_names=custom_loss_names)
 
     @lru_cache()
     def _preprocess_result(self) -> List[PreprocessResponse]:
@@ -173,7 +186,7 @@ class DatasetLoader:
         return preprocess_result
 
     def _get_dataset_handlers(
-            self, handlers: Iterable[DatasetBaseHandler], state: DataStateEnum, idx: int) -> Dict[str, np.ndarray]:
+            self, handlers: Iterable[DatasetBaseHandler], state: DataStateEnum, idx: int) -> Dict[str, npt.NDArray[np.float32]]:
         result_agg = {}
         preprocess_result = self._preprocess_result()
         preprocess_state = preprocess_result[state]
@@ -183,11 +196,11 @@ class DatasetLoader:
             result_agg[handler_name] = handler_result
         return result_agg
 
-    def _get_inputs(self, state: DataStateEnum, idx: int) -> Dict[str, np.ndarray]:
+    def _get_inputs(self, state: DataStateEnum, idx: int) -> Dict[str, npt.NDArray[np.float32]]:
         return self._get_dataset_handlers(global_dataset_binder.setup_container.inputs, state, idx)
 
-    def _get_gt(self, state: DataStateEnum, idx: int) -> Dict[str, np.ndarray]:
+    def _get_gt(self, state: DataStateEnum, idx: int) -> Dict[str, npt.NDArray[np.float32]]:
         return self._get_dataset_handlers(global_dataset_binder.setup_container.ground_truths, state, idx)
 
-    def _get_metadata(self, state: DataStateEnum, idx: int) -> Dict[str, np.ndarray]:
+    def _get_metadata(self, state: DataStateEnum, idx: int) -> Dict[str, npt.NDArray[np.float32]]:
         return self._get_dataset_handlers(global_dataset_binder.setup_container.metadata, state, idx)
